#include <linux/kernel.h> /* We're doing kernel work */
#include <linux/module.h> /* Specifically, a module */
#include <linux/proc_fs.h> /* Necessary because we use proc fs */
#include <linux/seq_file.h> /* for seq_file */

#define PROC_NAME "iter"
#define dbg(fmt,args...)        printk("[%s]:%d => "fmt,__FUNCTION__,__LINE__,##args) 
#define DBG()   printk("[%s]:%d => \n",__FUNCTION__,__LINE__) 

static char *sampleArray[3]={"string1","string2","string3"};

/**
* This function is called at the beginning of a sequence.
* ie, when:
* 1.the /proc file is read (first time)
* 2.after the function stop (end of sequence)
*
*/
static void *my_seq_start(struct seq_file *s, loff_t *pos)
{
	//DBG();
	if (*pos>=3) return NULL;
	return (void *)((unsigned long) *pos+1);
}
/**
* This function is called after the beginning of a sequence.
* It's called untill the return is NULL (this ends the sequence).
*
*/
static void *my_seq_next(struct seq_file *s, void *v, loff_t *pos)
{
	++*pos;
	return my_seq_start(s, pos);
}
/**
* This function is called at the end of a sequence
*
*/
static void my_seq_stop(struct seq_file *s, void *v)
{
	//DBG();
/* nothing to do, we use a static value in start() */
}
/**
* This function is called for each "step" of a sequence
*
*/
static int my_seq_show(struct seq_file *s, void *v)
{
	
	//unsigned long n;
	int n;
	
	//DBG();
	n = (int)v-1;
	seq_printf(s,"sampleArray[%d]:%s\n",n,sampleArray[n]);
	return 0;
}
/**
* This structure gather "function" to manage the sequence
*
*/
static struct seq_operations my_seq_ops = {
	.start = my_seq_start,
	.next = my_seq_next,
	.stop = my_seq_stop,
	.show = my_seq_show
};
/**
* This function is called when the /proc file is open.
*
*/
static int my_open(struct inode *inode, struct file *file)
{
	//DBG();
	return seq_open(file, &my_seq_ops);
};
/**
* This structure gather "function" that manage the /proc file
*
*/
static struct file_operations my_file_ops = {
	.owner = THIS_MODULE,
	.open = my_open,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = seq_release
};
/**
* This function is called when the module is loaded
*
*/
int init_module(void)
{
	struct proc_dir_entry *entry;
	entry = create_proc_entry(PROC_NAME, 0, NULL);
	if (entry) {
	entry->proc_fops = &my_file_ops;
	}
	return 0;
}
/**
* This function is called when the module is unloaded.
*
*/
void cleanup_module(void)
{
	remove_proc_entry(PROC_NAME, NULL);
}

MODULE_DESCRIPTION("seq module");
MODULE_AUTHOR("Joey Cheng<jemicheng@gmail.com>");
MODULE_LICENSE("GPL");
MODULE_ALIAS("QT2410:seq module");
